@@ -19,7 +19,9 @@
 #endif
 
 #ifdef HAVE_FTRUNCATE
-#include <unistd.h>      /* contains prototype of UNIX file truncate fn  */
+#if defined(unix) || defined(__unix__)  || defined(__unix)
+#include <unistd.h>  /* needed for getcwd prototype on unix machines */
+#endif
 #endif
 
 #define IO_SEEK 0        /* last file I/O operation was a seek */
@@ -356,9 +358,10 @@ int file_truncate(int handle, LONGLONG filesize)
 
     fdesc = fileno(handleTable[handle].fileptr);
     ftruncate(fdesc, (OFF_T) filesize);
+    file_seek(handle, filesize);
 
     handleTable[handle].currentpos = filesize;
-    handleTable[handle].last_io_op = IO_WRITE;
+    handleTable[handle].last_io_op = IO_SEEK;
 
 #endif
 
