@@ -287,13 +287,13 @@ static int irafrdimage (
 {
     FILE *fd;
     char *bang;
-    int nax, naxis1 = 1, naxis2 = 1, naxis3 = 1, naxis4 = 1, npaxis1, npaxis2;
+    int nax = 1, naxis1 = 1, naxis2 = 1, naxis3 = 1, naxis4 = 1, npaxis1 = 1, npaxis2;
     int bitpix, bytepix, i;
     char *fitsheader, *image;
     int nbr, nbimage, nbaxis, nbl, nbx, nbdiff;
     char *pixheader;
     char *linebuff;
-    int imhver, lpixhead;
+    int imhver, lpixhead = 0;
     char pixname[SZ_IM2PIXFILE+1];
     char errmsg[81];
     size_t newfilesize;
@@ -407,7 +407,7 @@ static int irafrdimage (
 	for (i = 0; i < naxis2; i++) {
 	    nbl = fread (linebuff, 1, nbaxis, fd);
 	    nbr = nbr + nbl;
-	    nbx = fseek (fd, nbdiff, SEEK_CUR);
+	    nbx = fseek (fd, nbdiff, 1);
 	    linebuff = linebuff + nbaxis;
 	    }
 	}
@@ -698,13 +698,17 @@ static int iraftofits (
 	pixname = irafgetc2 (irafheader, IM_PIXFILE, SZ_IMPIXFILE);
     if (strncmp(pixname, "HDR", 3) == 0 ) {
 	newpixname = same_path (pixname, hdrname);
-	free (pixname);
-	pixname = newpixname;
+        if (newpixname) {
+          free (pixname);
+          pixname = newpixname;
+	  }
 	}
     if (strchr (pixname, '/') == NULL && strchr (pixname, '$') == NULL) {
 	newpixname = same_path (pixname, hdrname);
-	free (pixname);
-	pixname = newpixname;
+        if (newpixname) {
+          free (pixname);
+          pixname = newpixname;
+	  }
 	}
 	
     if ((bang = strchr (pixname, '!')) != NULL )
@@ -846,7 +850,14 @@ char	*hdrname)	/* IRAF image header file pathname */
     int len;
     char *newpixname;
 
-    newpixname = (char *) calloc (SZ_IM2PIXFILE, sizeof (char));
+/*  WDP - 10/16/2007 - increased allocation to avoid possible overflow */
+/*    newpixname = (char *) calloc (SZ_IM2PIXFILE, sizeof (char)); */
+
+    newpixname = (char *) calloc (2*SZ_IM2PIXFILE+1, sizeof (char));
+    if (newpixname == NULL) {
+            ffpmsg("iraffits same_path: Cannot alloc memory for newpixname");
+	    return (NULL);
+	}
 
     /* Pixel file is in same directory as header */
     if (strncmp(pixname, "HDR$", 4) == 0 ) {
